@@ -193,10 +193,6 @@
     ,@'(*current-catch-block*
         *current-unwind-protect-block*)
 
-    ;; sb-safepoint in addition to accessing this symbol via TLS,
-    ;; uses the symbol itself as a value. Kinda weird.
-    #+(and sb-safepoint sb-thread) *in-without-gcing*
-
     #+immobile-space *immobile-freelist* ; not per-thread (yet...)
 
     ;; stack pointers
@@ -209,8 +205,9 @@
     ;; threading support
     #+sb-thread ,@'(sb-thread::*starting-threads* *free-tls-index*)
 
-    ;; dynamic runtime linking support
-    #+linkage-table +required-foreign-symbols+
+    ;; runtime linking of lisp->C calls (regardless of whether
+    ;; the C function is in a dynamic shared object or not)
+    +required-foreign-symbols+
 
     ;;; The following symbols aren't strictly required to be static
     ;;; - they are not accessed from C - but we make them static in order
