@@ -17,7 +17,8 @@
   (:export "GRAB-CONDITION" "ASSERT-ERROR"
            "HAS-ERROR?" "IS" "ASSERTOID"
            "ASSERT-SIGNAL" "ASSERT-NO-SIGNAL"
-           "EQUAL-MOD-GENSYMS"))
+           "LEGACY-EVAL-P"
+           "EQUAL-MOD-GENSYMS" "CHECK-FUNCTION-EVALUATION-ORDER"))
 
 (cl:in-package "ASSERTOID")
 
@@ -29,7 +30,12 @@
   `(typep (nth-value 1 (ignore-errors ,form)) ',error-subtype-spec))
 
 (defmacro assert-error (form &optional (error-subtype-spec 'error))
-  `(assert (typep (nth-value 1 (ignore-errors ,form)) ',error-subtype-spec)))
+  `(let ((result (multiple-value-list (ignore-errors ,form))))
+     (assert (typep (second result) ',error-subtype-spec)
+             ()  "~s returned ~s without signalling ~s"
+             ',form
+             result
+             ',error-subtype-spec)))
 
 (defun %assert-signal (thunk condition-type
                        expected-min-count expected-max-count)
@@ -191,3 +197,26 @@
                      (t ; strings, numbers
                       (funcall pred a b)))))
       (recurse a b))))
+
+(defun legacy-eval-p ()
+  (and (eq sb-ext:*evaluator-mode* :interpret)
+       (find-package "SB-EVAL")))
+
+(defmacro check-function-evaluation-order (form)
+  (let ((evals (gensym "EVALS"))
+        expected)
+    `(let ((,evals))
+       (multiple-value-prog1
+           (,(car form)
+            ,@(loop for i from 0
+                    for arg in (cdr form)
+                    collect `(progn
+                               (push ,i ,evals)
+                               ,arg)
+                    do
+                    (push i expected)))
+         (assert (equal ,evals ',expected)
+                 () 'simple-error
+                 :format-control "Bad evaluation order of ~s:~% ~s"
+                 :format-arguments (list ',form
+                                         (reverse ,evals)))))))
