@@ -99,7 +99,7 @@
 ;;; bug 379
 (with-test (:name (trace :encapsulate nil)
             :fails-on (or (and :ppc (not :linux)) :sparc :arm64)
-            :broken-on (or :hppa :freebsd))
+            :broken-on (or :freebsd))
   (let ((output (with-traced-function (trace-this :encapsulate nil)
                   (assert (eq 'ok (trace-this))))))
     (assert (search "TRACE-THIS" output))
@@ -107,7 +107,7 @@
 
 (with-test (:name (trace :encapsulate nil :recursive)
             :fails-on (or (and :ppc (not :linux)) :sparc :arm64)
-            :broken-on (or :hppa :freebsd))
+            :broken-on (or :freebsd))
   (let ((output (with-traced-function (trace-fact :encapsulate nil)
                   (assert (= 120 (trace-fact 5))))))
     (assert (search "TRACE-FACT" output))
@@ -168,7 +168,7 @@
     (untrace)
     (assert (>= (count #\Newline (get-output-stream-string s)) 4))))
 
-(with-test (:name :bug-310175 :fails-on (not :stack-allocatable-lists))
+(with-test (:name :bug-310175)
   ;; KLUDGE: Not all DX-enabled platforms DX CONS, and the compiler
   ;; transforms two-arg-LIST* (and one-arg-LIST) to CONS.  Therefore,
   ;; use two-arg-LIST, which should get through to VOP LIST, and thus
@@ -687,3 +687,16 @@
                      (format t "~x -> ~d (~a)~%"
                              ptr index (sb-kernel:make-lisp-obj tagged-fun))))))
             (assert (= count n))))))))
+
+(with-test (:name :repeatable-fasl)
+  (with-scratch-file (output1 "fasl")
+    (compile-file "bug-414.lisp" ; compile this file, why not
+                  ::output-file output1 :verbose nil :print nil)
+    (with-scratch-file (output2 "fasl")
+      (compile-file "bug-414.lisp" ; compile this file, why not
+                    ::output-file output2 :verbose nil :print nil)
+      (with-open-file (fasl1 output1 :element-type '(unsigned-byte 8))
+        (with-open-file (fasl2 output2 :element-type '(unsigned-byte 8))
+          (assert (= (file-length fasl1) (file-length fasl2)))
+          (loop repeat (file-length fasl1)
+                do (assert (= (read-byte fasl1) (read-byte fasl2)))))))))
