@@ -261,8 +261,7 @@
 ;;; which has two different freelists - one of cells that REMHASH has made available
 ;;; and one of cells that GC has marked as empty. Since we no longer inhibit GC
 ;;; during table operations, we need to give GC a list of its own to manipulate.
-(with-test (:name (hash-table :gc-smashed-cell-list)
-            :fails-on (and :sb-safepoint :x86 (not :win32)))
+(with-test (:name (hash-table :gc-smashed-cell-list))
   (flet ((f ()
            (dotimes (i 20000) (setf (gethash i *tbl*) (- i)))
            (setf (gethash (cons 1 2) *tbl*) 'foolz)
@@ -272,6 +271,7 @@
     ;; Ensure the values remain outside of the stack pointer for scrub-control-stack to work
     (declare (notinline f))
     (f))
+  (eval nil)
   (sb-sys:scrub-control-stack)
   (gc)
   ;; There were 20 items REMHASHed plus the freelist contains a pointer
